package it.amattioli.applicate.commands;

import it.amattioli.dominate.util.PropertyChangeEmitter;

import java.util.List;

/**
 * An editor for a list of entities. Implementations of this interface maintain a
 * list of editors, one for each entity contained in the editing list. A new entity
 * can be added to the list using the {@link #addRow()} method. The {@link #deleteRow()}
 * method, instead, can be used to remove an entity from the editing list.
 * 
 * @param <T>
 */
public interface ListEditor<T> extends PropertyChangeEmitter {

	/**
	 * Set the list that has to be edited by this editor.
	 * 
	 * @param editingList the list that has to be edited by this editor
	 */
	public void setEditingList(List<T> editingList);

	/**
	 * Retrieve the list that is being edited by this editor.
	 * 
	 * @return the list that is being edited by this editor
	 */
	public List<T> getEditingList();
	
	/**
	 * Set the behavior of this editor when the editing list is empty.
	 * If the emptyRow property is set to true the editor will automatically
	 * create a new row so the list will never be empty. If the property is
	 * set to false the list can be empty, so no new entity is created.
	 *  
	 * @param emptyRow
	 */
	public void setEmptyRow(boolean emptyRow);

	public boolean isEmptyRow();

	public void select(int idx);

	public int getSelectedIndex();

	public T getSelectedObject();

	/**
	 * Retrieves the editor for the entity with the given index.
	 * 
	 * @param index
	 * @return
	 */
	public BeanEditor<T> getElementEditor(int index);

	/**
	 * Create a new entity (and a corresponding editor) at the end of the list.
	 * 
	 * @param param a list of param that can be necessary for building a new entity instance
	 * @return the new entity
	 */
	public T addRow(Object... param);

	/**
	 * Delete from the list the entity (and the corresponding editor) at given index.
	 * 
	 * @return
	 */
	public T deleteRow();

}