package it.amattioli.applicate.commands;

import it.amattioli.applicate.context.hibernate.LongRunningContext;

import java.lang.reflect.Method;

import org.hibernate.Session;
import org.hibernate.StaleObjectStateException;
import org.hibernate.Transaction;
import org.hibernate.exception.JDBCConnectionException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import net.sf.cglib.proxy.Enhancer;
import net.sf.cglib.proxy.MethodProxy;

@Deprecated
public class TransactionalCommandContext extends LongRunningContext {
	private static final Logger logger = LoggerFactory.getLogger(TransactionalCommandContext.class);
	private Command decorated;
	private CommandEventSupport cmdEvtSupport = new CommandEventSupport();
	
	public static <T extends Command> T newTransaction(T command) {
		Enhancer enhancer = new Enhancer();
        enhancer.setSuperclass(command.getClass());
        enhancer.setCallback(new TransactionalCommandContext(command));
        T result = (T)enhancer.create();
        setDependenciesContextOn(command, result);
		return result;
	}

    public TransactionalCommandContext(Command decorated) {
    	super(decorated);
        this.decorated = decorated;
    }

    private void doCommand(Command invocatedCmd) throws ApplicationException {
    	logger.debug("Beginning Transaction");
        Transaction tx = getSessionManager().getSession(Session.class).beginTransaction();
        CommandEvent evt = null;
        try {
        	decorated.doCommand();
            logger.debug("Flushing transaction");
            tx.commit();
            logger.debug("Transaction committed");
            evt = new CommandEvent(invocatedCmd, CommandResult.SUCCESSFUL);
        } catch (StaleObjectStateException e) {
        	tx.rollback();
            logger.debug("Transaction Rolled-back",e);
            evt = new CommandEvent(invocatedCmd, CommandResult.UNSUCCESSFUL, e);
            throw new ConcurrencyException(e);
        } catch (JDBCConnectionException e) {
        	logger.debug("Database connection problems, transaction cannot be committed nor rolled back");
        	evt = new CommandEvent(invocatedCmd, CommandResult.UNSUCCESSFUL, e);
            throw e;
        } catch (ApplicationException e) {
        	tx.rollback();
            logger.debug("Transaction Rolled-back");
            evt = new CommandEvent(invocatedCmd, CommandResult.UNSUCCESSFUL, e);
            throw e;
        } catch (RuntimeException e) {
            tx.rollback();
            logger.debug("Transaction Rolled-back");
            evt = new CommandEvent(invocatedCmd, CommandResult.UNSUCCESSFUL, e);
            throw e;
        } finally {
        	cmdEvtSupport.fireCommandEvent(evt);
        	if (invocatedCmd instanceof Resettable && ((Resettable)invocatedCmd).toBeReset()) {
        		reset();
        	} else {
        		getSessionManager().release();
        	}
        }
    }

    private void cancelCommand(Command invocatedCmd) {
    	try {
    		decorated.cancelCommand();
	    } finally {
			cmdEvtSupport.fireCommandEvent(new CommandEvent(invocatedCmd, CommandResult.CANCELLED));
			if (invocatedCmd instanceof Resettable && ((Resettable)invocatedCmd).toBeReset()) {
        		reset();
        	} else {
        		getSessionManager().release();
        	}
		}
    }

    private void addCommandListener(CommandListener listener) {
    	logger.debug("Registering transactional command listener " + listener);
    	cmdEvtSupport.addListener(listener);
    }

    private void addCommandListener(CommandListener listener, CommandResult... results) {
    	logger.debug("Registering transactional command listener " + listener);
    	cmdEvtSupport.addListener(listener, results);
    }
    
    private void reset() {
    	getSessionManager().reset();
		((Resettable)decorated).reset();
    }

    /*
	private static final Class<?>[] ADD_GENERIC_LSNR_PARMS =
    	new Class<?>[] {CommandListener.class};
	private static final Class<?>[] ADD_SPECIFIC_LSNR_PARMS =
    	new Class<?>[] {CommandListener.class, CommandResult[].class};
	*/
	public Object perform(Object object, Method method, Object[] args, MethodProxy methodProxy) throws Throwable {
		if ("doCommand".equals(method.getName())) {
			doCommand((Command) object);
			return null;
		} else if ("cancelCommand".equals(method.getName())) {
			cancelCommand((Command)object);
			return null;
		} else if ("addCommandListener".equals(method.getName())
					&& method.getParameterTypes().length == 2) {  //ADD_SPECIFIC_LSNR_PARMS.equals(method.getParameterTypes())) {
			addCommandListener((CommandListener) args[0], (CommandResult[]) args[1]);
			return null;
		} else if ("addCommandListener".equals(method.getName())
					&& method.getParameterTypes().length == 1) {  //ADD_GENERIC_LSNR_PARMS.equals(method.getParameterTypes())) {
			addCommandListener((CommandListener) args[0]);
			return null;
		} else if ("reset".equals(method.getName()) && object instanceof Resettable) {
			reset();
			return null;
		} else {
			return method.invoke(decorated, args);
		}
	}

}
