package it.amattioli.applicate.sessions;

import java.util.Collection;
import java.util.HashSet;

public class CompositeServiceFactory  implements ServiceFactory {
    private Collection<ServiceFactory> factories = new HashSet<ServiceFactory>();

    private ServiceFactory findFactory(String serviceName) {
        for (ServiceFactory curr : factories) {
            if (curr.knowsService(serviceName)) {
                return curr;
            }
        }
        return null;
    }

    @Override
    public Object createService(String serviceName) {
        ServiceFactory factory = findFactory(serviceName);
        if (factory == null) {
        	throw new UnknownServiceException(serviceName);
        }
        return factory.createService(serviceName);
    }

    @Override
    public boolean knowsService(String serviceName) {
        return findFactory(serviceName) != null;
    }

    public void addFactory(ServiceFactory factory) {
    	factories.add(factory);
    }
}
